require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
	class Game
		attr_reader :title

		def initialize(title)
			@title = title
			@players = []
		end

		def add_player(player)
			@players << player		
		end

		def load_players(from_file)
			File.readlines(from_file).each do |line|
				add_player(Player.from_csv(line))
			end
		end

		def play(rounds)
			puts "There are #{@players.size} players in #{@title}: "
			
			@players.each do |player|
				puts player
			end

			treasures = TreasureTrove::TREASURES
			puts "\nThere are #{treasures.size} treasures to be found:"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points} points"
			end

			1.upto(rounds) do |round|
				puts "\nRound #{round}:"
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end
		end

		def print_stats
			strong_players, wimpy_players = @players.partition { |player| player.strong? }

			puts "\n#{@title} statistics:"

			puts "\n#{strong_players.size} strong players:"
			strong_players.each do |player|
				print_name_and_health(player)
			end

			puts "\n#{wimpy_players.size} wimpy players:"
			wimpy_players.each do |player|
				print_name_and_health(player)
			end

			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "\n#{@title} High Scores:"
			@players.sort.each do |player|
				puts high_score_entry(player)
			end
		end

		def save_high_scores(to_file="high_scores.txt")
			File.open(to_file, "w") do |file|
				file.puts "#{@title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(15, '.')
			formatted_score = player.score.to_s.rjust(10, '.')
			"#{formatted_name}#{formatted_score}"
		end

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end
	end
end

	# example code
	if __FILE__ == $0
		player1 = Player.new("moe")
		player2 = Player.new("Larry", 60)
		player3 = Player.new("Curly", 125)

		knuckleheads =  StudioGame::Game.new('Knuckleheads')
		knuckleheads.add_player(player1)
		knuckleheads.add_player(player2)
		knuckleheads.add_player(player3)
		knuckleheads.play(3)
		knuckleheads.print_stats
	end
